@@ -80,7 +80,7 @@ import qualified Data.Map.Lazy as Map
 import qualified Data.HashMap.Lazy as HashMap
 import           Data.List (nub, (\\), sortBy, intercalate)
 import           Data.Maybe
-import           Data.Monoid hiding (Product)
+import           Data.Semigroup (Semigroup(..))
 import           Data.Ord
 import           Data.Text (Text)
 import qualified Data.Text as T
@@ -124,6 +124,7 @@ package name version = Package {
   , packageExtraSourceFiles = []
   , packageExtraDocFiles = []
   , packageDataFiles = []
+  , packageDataDir = Nothing
   , packageSourceRepository = Nothing
   , packageCustomSetup = Nothing
   , packageLibrary = Nothing
@@ -162,7 +163,7 @@ packageDependencies Package{..} = nub . sortBy (comparing (lexicographically . f
     deps xs = [(name, version) | (name, version) <- (Map.toList . unDependencies . sectionDependencies) xs]
 
 section :: a -> Section a
-section a = Section a [] mempty [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] Nothing [] mempty []
+section a = Section a [] mempty [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] Nothing [] mempty []
 
 packageConfig :: FilePath
 packageConfig = "package.yaml"
@@ -183,7 +184,10 @@ data LibrarySection = LibrarySection {
 
 instance Monoid LibrarySection where
   mempty = LibrarySection Nothing Nothing Nothing Nothing Nothing Nothing Nothing
-  mappend a b = LibrarySection {
+  mappend = (<>)
+
+instance Semigroup LibrarySection where
+  a <> b = LibrarySection {
       librarySectionExposed = librarySectionExposed b <|> librarySectionExposed a
     , librarySectionExposedModules = librarySectionExposedModules a <> librarySectionExposedModules b
     , librarySectionGeneratedExposedModules = librarySectionGeneratedExposedModules a <> librarySectionGeneratedExposedModules b
@@ -201,7 +205,10 @@ data ExecutableSection = ExecutableSection {
 
 instance Monoid ExecutableSection where
   mempty = ExecutableSection Nothing Nothing Nothing
-  mappend a b = ExecutableSection {
+  mappend = (<>)
+
+instance Semigroup ExecutableSection where
+  a <> b = ExecutableSection {
       executableSectionMain = executableSectionMain b <|> executableSectionMain a
     , executableSectionOtherModules = executableSectionOtherModules a <> executableSectionOtherModules b
     , executableSectionGeneratedOtherModules = executableSectionGeneratedOtherModules a <> executableSectionGeneratedOtherModules b
@@ -234,7 +241,7 @@ instance FromValue Verbatim where
     Object _ -> VerbatimObject <$> fromValue v
     _ -> typeMismatch (formatOrList ["String", "Object"]) v
 
-data CommonOptions cSources jsSources a = CommonOptions {
+data CommonOptions cSources cxxSources jsSources a = CommonOptions {
   commonOptionsSourceDirs :: Maybe (List FilePath)
 , commonOptionsDependencies :: Maybe Dependencies
 , commonOptionsPkgConfigDependencies :: Maybe (List String)
@@ -246,6 +253,8 @@ data CommonOptions cSources jsSources a = CommonOptions {
 , commonOptionsCppOptions :: Maybe (List CppOption)
 , commonOptionsCcOptions :: Maybe (List CcOption)
 , commonOptionsCSources :: cSources
+, commonOptionsCxxOptions :: Maybe (List CxxOption)
+, commonOptionsCxxSources :: cxxSources
 , commonOptionsJsSources :: jsSources
 , commonOptionsExtraLibDirs :: Maybe (List FilePath)
 , commonOptionsExtraLibraries :: Maybe (List FilePath)
@@ -255,15 +264,15 @@ data CommonOptions cSources jsSources a = CommonOptions {
 , commonOptionsInstallIncludes :: Maybe (List FilePath)
 , commonOptionsLdOptions :: Maybe (List LdOption)
 , commonOptionsBuildable :: Maybe Bool
-, commonOptionsWhen :: Maybe (List (ConditionalSection cSources jsSources a))
+, commonOptionsWhen :: Maybe (List (ConditionalSection cSources cxxSources jsSources a))
 , commonOptionsBuildTools :: Maybe Dependencies
 , commonOptionsVerbatim :: Maybe (List Verbatim)
 } deriving (Functor, Generic)
 
-type ParseCommonOptions = CommonOptions ParseCSources ParseJsSources
+type ParseCommonOptions = CommonOptions ParseCSources ParseCxxSources ParseJsSources
 instance FromValue a => FromValue (ParseCommonOptions a)
 
-instance (Monoid cSources, Monoid jsSources) => Monoid (CommonOptions cSources jsSources a) where
+instance (Semigroup cSources, Semigroup cxxSources, Semigroup jsSources, Monoid cSources, Monoid cxxSources, Monoid jsSources) => Monoid (CommonOptions cSources cxxSources jsSources a) where
   mempty = CommonOptions {
     commonOptionsSourceDirs = Nothing
   , commonOptionsDependencies = Nothing
@@ -276,6 +285,8 @@ instance (Monoid cSources, Monoid jsSources) => Monoid (CommonOptions cSources j
   , commonOptionsCppOptions = Nothing
   , commonOptionsCcOptions = Nothing
   , commonOptionsCSources = mempty
+  , commonOptionsCxxOptions = Nothing
+  , commonOptionsCxxSources = mempty
   , commonOptionsJsSources = mempty
   , commonOptionsExtraLibDirs = Nothing
   , commonOptionsExtraLibraries = Nothing
@@ -289,7 +300,10 @@ instance (Monoid cSources, Monoid jsSources) => Monoid (CommonOptions cSources j
   , commonOptionsBuildTools = Nothing
   , commonOptionsVerbatim = Nothing
   }
-  mappend a b = CommonOptions {
+  mappend = (<>)
+
+instance (Semigroup cSources, Semigroup cxxSources, Semigroup jsSources) => Semigroup (CommonOptions cSources cxxSources jsSources a) where
+  a <> b = CommonOptions {
     commonOptionsSourceDirs = commonOptionsSourceDirs a <> commonOptionsSourceDirs b
   , commonOptionsDependencies = commonOptionsDependencies b <> commonOptionsDependencies a
   , commonOptionsPkgConfigDependencies = commonOptionsPkgConfigDependencies a <> commonOptionsPkgConfigDependencies b
@@ -301,6 +315,8 @@ instance (Monoid cSources, Monoid jsSources) => Monoid (CommonOptions cSources j
   , commonOptionsCppOptions = commonOptionsCppOptions a <> commonOptionsCppOptions b
   , commonOptionsCcOptions = commonOptionsCcOptions a <> commonOptionsCcOptions b
   , commonOptionsCSources = commonOptionsCSources a <> commonOptionsCSources b
+  , commonOptionsCxxOptions = commonOptionsCxxOptions a <> commonOptionsCxxOptions b
+  , commonOptionsCxxSources = commonOptionsCxxSources a <> commonOptionsCxxSources b
   , commonOptionsJsSources = commonOptionsJsSources a <> commonOptionsJsSources b
   , commonOptionsExtraLibDirs = commonOptionsExtraLibDirs a <> commonOptionsExtraLibDirs b
   , commonOptionsExtraLibraries = commonOptionsExtraLibraries a <> commonOptionsExtraLibraries b
@@ -316,35 +332,40 @@ instance (Monoid cSources, Monoid jsSources) => Monoid (CommonOptions cSources j
   }
 
 type ParseCSources = Maybe (List FilePath)
+type ParseCxxSources = Maybe (List FilePath)
 type ParseJsSources = Maybe (List FilePath)
 
 type CSources = [FilePath]
+type CxxSources = [FilePath]
 type JsSources = [FilePath]
 
-type WithCommonOptions cSources jsSources a = Product (CommonOptions cSources jsSources a) a
+type WithCommonOptions cSources cxxSources jsSources a = Product (CommonOptions cSources cxxSources jsSources a) a
 
-data Traverse m cSources cSources_ jsSources jsSources_ = Traverse {
+data Traverse m cSources cSources_ cxxSources cxxSources_ jsSources jsSources_ = Traverse {
   traverseCSources :: cSources -> m cSources_
+, traverseCxxSources :: cxxSources -> m cxxSources_
 , traverseJsSources :: jsSources -> m jsSources_
 }
 
-type Traversal t = forall m cSources cSources_ jsSources jsSources_. Monad m
-  => Traverse m cSources cSources_ jsSources jsSources_
-  -> t cSources jsSources
-  -> m (t cSources_ jsSources_)
+type Traversal t = forall m cSources cSources_ cxxSources cxxSources_ jsSources jsSources_. Monad m
+  => Traverse m cSources cSources_ cxxSources cxxSources_ jsSources jsSources_
+  -> t cSources cxxSources jsSources
+  -> m (t cSources_ cxxSources_ jsSources_)
 
-type Traversal_ t = forall m cSources cSources_ jsSources jsSources_ a. Monad m
-  => Traverse m cSources cSources_ jsSources jsSources_
-  -> t cSources jsSources a
-  -> m (t cSources_ jsSources_ a)
+type Traversal_ t = forall m cSources cSources_ cxxSources cxxSources_ jsSources jsSources_ a. Monad m
+  => Traverse m cSources cSources_ cxxSources cxxSources_ jsSources jsSources_
+  -> t cSources cxxSources jsSources a
+  -> m (t cSources_ cxxSources_ jsSources_ a)
 
 traverseCommonOptions :: Traversal_ CommonOptions
 traverseCommonOptions t@Traverse{..} c@CommonOptions{..} = do
   cSources <- traverseCSources commonOptionsCSources
+  cxxSources <- traverseCxxSources commonOptionsCxxSources
   jsSources <- traverseJsSources commonOptionsJsSources
   xs <- traverse (traverse (traverseConditionalSection t)) commonOptionsWhen
   return c {
       commonOptionsCSources = cSources
+    , commonOptionsCxxSources = cxxSources
     , commonOptionsJsSources = jsSources
     , commonOptionsWhen = xs
     }
@@ -363,16 +384,16 @@ traverseThenElse t@Traverse{..} c@ThenElse{..} = do
 traverseWithCommonOptions :: Traversal_ WithCommonOptions
 traverseWithCommonOptions t = bitraverse (traverseCommonOptions t) return
 
-data ConditionalSection cSources jsSources a =
-    ThenElseConditional (Product (ThenElse cSources jsSources a) Condition)
-  | FlatConditional (Product (WithCommonOptions cSources jsSources a) Condition)
+data ConditionalSection cSources cxxSources jsSources a =
+    ThenElseConditional (Product (ThenElse cSources cxxSources jsSources a) Condition)
+  | FlatConditional (Product (WithCommonOptions cSources cxxSources jsSources a) Condition)
 
-instance Functor (ConditionalSection cSources jsSources) where
+instance Functor (ConditionalSection cSources cxxSources jsSources) where
   fmap f = \ case
     ThenElseConditional c -> ThenElseConditional (first (fmap f) c)
     FlatConditional c -> FlatConditional (first (bimap (fmap f) f) c)
 
-type ParseConditionalSection = ConditionalSection ParseCSources ParseJsSources
+type ParseConditionalSection = ConditionalSection ParseCSources ParseCxxSources ParseJsSources
 
 instance FromValue a => FromValue (ParseConditionalSection a) where
   fromValue v
@@ -397,17 +418,17 @@ instance FromValue Cond where
     Bool False -> return (Cond "false")
     _ -> typeMismatch "Boolean or String" v
 
-data ThenElse cSources jsSources a = ThenElse {
-  thenElseThen :: WithCommonOptions cSources jsSources a
-, thenElseElse :: WithCommonOptions cSources jsSources a
+data ThenElse cSources cxxSources jsSources a = ThenElse {
+  thenElseThen :: WithCommonOptions cSources cxxSources jsSources a
+, thenElseElse :: WithCommonOptions cSources cxxSources jsSources a
 } deriving Generic
 
-instance Functor (ThenElse cSources jsSources) where
+instance Functor (ThenElse cSources cxxSources jsSources) where
   fmap f c@ThenElse{..} = c{thenElseThen = map_ thenElseThen, thenElseElse = map_ thenElseElse}
     where
       map_ = bimap (fmap f) f
 
-type ParseThenElse = ThenElse ParseCSources ParseJsSources
+type ParseThenElse = ThenElse ParseCSources ParseCxxSources ParseJsSources
 
 instance FromValue a => FromValue (ParseThenElse a)
 
@@ -416,7 +437,10 @@ data Empty = Empty
 
 instance Monoid Empty where
   mempty = Empty
-  mappend Empty Empty = Empty
+  mappend = (<>)
+
+instance Semigroup Empty where
+  Empty <> Empty = Empty
 
 instance FromValue Empty where
   fromValue _ = return Empty
@@ -444,17 +468,15 @@ formatOrList xs = case reverse xs of
   y : x : [] -> x ++ " or " ++ y
   x : ys@(_:_:_) -> intercalate ", " . reverse $ ("or " ++ x) : ys
 
-type SectionConfigWithDefaluts cSources jsSources a = Product DefaultsConfig (WithCommonOptions cSources jsSources a)
+type SectionConfigWithDefaluts cSources cxxSources jsSources a = Product DefaultsConfig (WithCommonOptions cSources cxxSources jsSources a)
 
-type PackageConfigWithDefaults cSources jsSources = PackageConfig_
-  (SectionConfigWithDefaluts cSources jsSources LibrarySection)
-  (SectionConfigWithDefaluts cSources jsSources ExecutableSection)
-  cSources jsSources
+type PackageConfigWithDefaults cSources cxxSources jsSources = PackageConfig_
+  (SectionConfigWithDefaluts cSources cxxSources jsSources LibrarySection)
+  (SectionConfigWithDefaluts cSources cxxSources jsSources ExecutableSection)
 
-type PackageConfig cSources jsSources = PackageConfig_
-  (WithCommonOptions cSources jsSources LibrarySection)
-  (WithCommonOptions cSources jsSources ExecutableSection)
-  cSources jsSources
+type PackageConfig cSources cxxSources jsSources = PackageConfig_
+  (WithCommonOptions cSources cxxSources jsSources LibrarySection)
+  (WithCommonOptions cSources cxxSources jsSources ExecutableSection)
 
 data PackageVersion = PackageVersion {unPackageVersion :: String}
 
@@ -464,7 +486,7 @@ instance FromValue PackageVersion where
     String s -> return (T.unpack s)
     _ -> typeMismatch "Number or String" v
 
-data PackageConfig_ library executable cSources jsSources = PackageConfig {
+data PackageConfig_ library executable = PackageConfig {
   packageConfigName :: Maybe String
 , packageConfigVersion :: Maybe PackageVersion
 , packageConfigSynopsis :: Maybe String
@@ -484,6 +506,7 @@ data PackageConfig_ library executable cSources jsSources = PackageConfig {
 , packageConfigExtraSourceFiles :: Maybe (List FilePath)
 , packageConfigExtraDocFiles :: Maybe (List FilePath)
 , packageConfigDataFiles :: Maybe (List FilePath)
+, packageConfigDataDir :: Maybe FilePath
 , packageConfigGithub :: Maybe Text
 , packageConfigGit :: Maybe String
 , packageConfigCustomSetup :: Maybe CustomSetupSection
@@ -518,7 +541,7 @@ traversePackageConfig t@Traverse{..} p@PackageConfig{..} = do
   where
     traverseNamedConfigs = traverse . traverse . traverseWithCommonOptions
 
-type ParsePackageConfig = PackageConfigWithDefaults ParseCSources ParseJsSources
+type ParsePackageConfig = PackageConfigWithDefaults ParseCSources ParseCxxSources ParseJsSources
 
 instance FromValue ParsePackageConfig
 
@@ -588,6 +611,7 @@ data Package = Package {
 , packageExtraSourceFiles :: [FilePath]
 , packageExtraDocFiles :: [FilePath]
 , packageDataFiles :: [FilePath]
+, packageDataDir :: Maybe FilePath
 , packageSourceRepository :: Maybe SourceRepository
 , packageCustomSetup :: Maybe CustomSetup
 , packageLibrary :: Maybe (Section Library)
@@ -630,6 +654,8 @@ data Section a = Section {
 , sectionCppOptions :: [CppOption]
 , sectionCcOptions :: [CcOption]
 , sectionCSources :: [FilePath]
+, sectionCxxOptions :: [CxxOption]
+, sectionCxxSources :: [FilePath]
 , sectionJsSources :: [FilePath]
 , sectionExtraLibDirs :: [FilePath]
 , sectionExtraLibraries :: [FilePath]
@@ -671,18 +697,18 @@ data SourceRepository = SourceRepository {
 , sourceRepositorySubdir :: Maybe String
 } deriving (Eq, Show)
 
-type Config cSources jsSources =
-  Product (CommonOptions cSources jsSources Empty) (PackageConfig cSources jsSources)
+type Config cSources cxxSources jsSources =
+  Product (CommonOptions cSources cxxSources jsSources Empty) (PackageConfig cSources cxxSources jsSources)
 
 traverseConfig :: Traversal Config
 traverseConfig t = bitraverse (traverseCommonOptions t) (traversePackageConfig t)
 
 type ConfigWithDefaults = Product
   (CommonOptionsWithDefaults Empty)
-  (PackageConfigWithDefaults ParseCSources ParseJsSources)
+  (PackageConfigWithDefaults ParseCSources ParseCxxSources ParseJsSources)
 
-type CommonOptionsWithDefaults a = Product DefaultsConfig (CommonOptions ParseCSources ParseJsSources a)
-type WithCommonOptionsWithDefaults a = Product DefaultsConfig (WithCommonOptions ParseCSources ParseJsSources a)
+type CommonOptionsWithDefaults a = Product DefaultsConfig (CommonOptions ParseCSources ParseCxxSources ParseJsSources a)
+type WithCommonOptionsWithDefaults a = Product DefaultsConfig (WithCommonOptions ParseCSources ParseCxxSources ParseJsSources a)
 
 toPackage :: FilePath -> FilePath -> ConfigWithDefaults -> Warnings (Errors IO) Package
 toPackage userDataDir dir =
@@ -694,22 +720,22 @@ expandDefaultsInConfig
   :: FilePath
   -> FilePath
   -> ConfigWithDefaults
-  -> Warnings (Errors IO) (Config ParseCSources ParseJsSources)
+  -> Warnings (Errors IO) (Config ParseCSources ParseCxxSources ParseJsSources)
 expandDefaultsInConfig userDataDir dir = bitraverse (expandGlobalDefaults userDataDir dir) (expandSectionDefaults userDataDir dir)
 
 expandGlobalDefaults
   :: FilePath
   -> FilePath
   -> CommonOptionsWithDefaults Empty
-  -> Warnings (Errors IO) (CommonOptions ParseCSources ParseJsSources Empty)
+  -> Warnings (Errors IO) (CommonOptions ParseCSources ParseCxxSources ParseJsSources Empty)
 expandGlobalDefaults userDataDir dir = do
   fmap (`Product` Empty) >>> expandDefaults userDataDir dir >=> \ (Product c Empty) -> return c
 
 expandSectionDefaults
   :: FilePath
   -> FilePath
-  -> PackageConfigWithDefaults ParseCSources ParseJsSources
-  -> Warnings (Errors IO) (PackageConfig ParseCSources ParseJsSources)
+  -> PackageConfigWithDefaults ParseCSources ParseCxxSources ParseJsSources
+  -> Warnings (Errors IO) (PackageConfig ParseCSources ParseCxxSources ParseJsSources)
 expandSectionDefaults userDataDir dir p@PackageConfig{..} = do
   library <- traverse (expandDefaults userDataDir dir) packageConfigLibrary
   internalLibraries <- traverse (traverse (expandDefaults userDataDir dir)) packageConfigInternalLibraries
@@ -727,27 +753,27 @@ expandSectionDefaults userDataDir dir p@PackageConfig{..} = do
     }
 
 expandDefaults
-  :: (FromValue a, Monoid a)
+  :: (FromValue a, Semigroup a, Monoid a)
   => FilePath
   -> FilePath
   -> WithCommonOptionsWithDefaults a
-  -> Warnings (Errors IO) (WithCommonOptions ParseCSources ParseJsSources a)
+  -> Warnings (Errors IO) (WithCommonOptions ParseCSources ParseCxxSources ParseJsSources a)
 expandDefaults userDataDir = expand []
   where
-    expand :: (FromValue a, Monoid a) =>
+    expand :: (FromValue a, Semigroup a, Monoid a) =>
          [FilePath]
       -> FilePath
       -> WithCommonOptionsWithDefaults a
-      -> Warnings (Errors IO) (WithCommonOptions ParseCSources ParseJsSources a)
+      -> Warnings (Errors IO) (WithCommonOptions ParseCSources ParseCxxSources ParseJsSources a)
     expand seen dir (Product DefaultsConfig{..} c) = do
       d <- mconcat <$> mapM (get seen dir) (fromMaybeList defaultsConfigDefaults)
       return (d <> c)
 
-    get :: forall a. (FromValue a, Monoid a) =>
+    get :: forall a. (FromValue a, Semigroup a, Monoid a) =>
          [FilePath]
       -> FilePath
       -> Defaults
-      -> Warnings (Errors IO) (WithCommonOptions ParseCSources ParseJsSources a)
+      -> Warnings (Errors IO) (WithCommonOptions ParseCSources ParseCxxSources ParseJsSources a)
     get seen dir defaults = do
       file <- lift $ ExceptT (ensure userDataDir dir defaults)
       seen_ <- lift (checkCycle seen file)
@@ -771,9 +797,9 @@ toExecutableMap name executables mExecutable = do
       return $ Just (Map.fromList [(name, executable)])
     Nothing -> return executables
 
-type GlobalOptions = CommonOptions CSources JsSources Empty
+type GlobalOptions = CommonOptions CSources CxxSources JsSources Empty
 
-toPackage_ :: MonadIO m => FilePath -> Product GlobalOptions (PackageConfig CSources JsSources) -> Warnings m Package
+toPackage_ :: MonadIO m => FilePath -> Product GlobalOptions (PackageConfig CSources CxxSources JsSources) -> Warnings m Package
 toPackage_ dir (Product g PackageConfig{..}) = do
   let
     globalVerbatim = commonOptionsVerbatim g
@@ -801,7 +827,10 @@ toPackage_ dir (Product g PackageConfig{..}) = do
 
   extraSourceFiles <- expandGlobs "extra-source-files" dir (fromMaybeList packageConfigExtraSourceFiles)
   extraDocFiles <- expandGlobs "extra-doc-files" dir (fromMaybeList packageConfigExtraDocFiles)
-  dataFiles <- expandGlobs "data-files" dir (fromMaybeList packageConfigDataFiles)
+
+  let dataBaseDir = maybe dir (dir </>) packageConfigDataDir
+
+  dataFiles <- expandGlobs "data-files" dataBaseDir (fromMaybeList packageConfigDataFiles)
 
   let defaultBuildType :: BuildType
       defaultBuildType = maybe Simple (const Custom) mCustomSetup
@@ -831,6 +860,7 @@ toPackage_ dir (Product g PackageConfig{..}) = do
       , packageExtraSourceFiles = extraSourceFiles
       , packageExtraDocFiles = extraDocFiles
       , packageDataFiles = dataFiles
+      , packageDataDir = packageConfigDataDir
       , packageSourceRepository = sourceRepository
       , packageCustomSetup = mCustomSetup
       , packageLibrary = mLibrary
@@ -893,9 +923,10 @@ toPackage_ dir (Product g PackageConfig{..}) = do
 expandForeignSources
   :: MonadIO m
   => FilePath
-  -> Traverse (Warnings m) ParseCSources CSources ParseJsSources JsSources
+  -> Traverse (Warnings m) ParseCSources CSources ParseCxxSources CxxSources ParseJsSources JsSources
 expandForeignSources dir = Traverse {
     traverseCSources = expand "c-sources"
+  , traverseCxxSources = expand "cxx-sources"
   , traverseJsSources = expand "js-sources"
   }
   where
@@ -955,7 +986,7 @@ inferModules dir packageName_ getMentionedModules getInferredModules fromData fr
         r = fromConfig pathsModule inferableModules conf
       return (outerModules ++ getInferredModules r, r)
 
-toLibrary :: FilePath -> String -> GlobalOptions -> WithCommonOptions CSources JsSources LibrarySection -> IO (Section Library)
+toLibrary :: FilePath -> String -> GlobalOptions -> WithCommonOptions CSources CxxSources JsSources LibrarySection -> IO (Section Library)
 toLibrary dir name globalOptions =
     inferModules dir name getMentionedLibraryModules getLibraryModules fromLibrarySectionTopLevel fromLibrarySectionInConditional
   . toSection (mempty <$ globalOptions)
@@ -997,17 +1028,17 @@ fromLibrarySectionPlain LibrarySection{..} = Library {
   , librarySignatures = fromMaybeList librarySectionSignatures
   }
 
-toInternalLibraries :: FilePath -> String -> GlobalOptions -> Maybe (Map String (WithCommonOptions CSources JsSources LibrarySection)) -> IO (Map String (Section Library))
+toInternalLibraries :: FilePath -> String -> GlobalOptions -> Maybe (Map String (WithCommonOptions CSources CxxSources JsSources LibrarySection)) -> IO (Map String (Section Library))
 toInternalLibraries dir packageName_ globalOptions = traverse (toLibrary dir packageName_ globalOptions) . fromMaybe mempty
 
-toExecutables :: FilePath -> String -> GlobalOptions -> Maybe (Map String (WithCommonOptions CSources JsSources ExecutableSection)) -> IO (Map String (Section Executable))
+toExecutables :: FilePath -> String -> GlobalOptions -> Maybe (Map String (WithCommonOptions CSources CxxSources JsSources ExecutableSection)) -> IO (Map String (Section Executable))
 toExecutables dir packageName_ globalOptions = traverse (toExecutable dir packageName_ globalOptions) . fromMaybe mempty
 
 getMentionedExecutableModules :: ExecutableSection -> [String]
 getMentionedExecutableModules (ExecutableSection main otherModules generatedModules)=
   maybe id (:) (main >>= toModule . splitDirectories) $ fromMaybeList (otherModules <> generatedModules)
 
-toExecutable :: FilePath -> String -> GlobalOptions -> WithCommonOptions CSources JsSources ExecutableSection -> IO (Section Executable)
+toExecutable :: FilePath -> String -> GlobalOptions -> WithCommonOptions CSources CxxSources JsSources ExecutableSection -> IO (Section Executable)
 toExecutable dir packageName_ globalOptions =
     inferModules dir packageName_ getMentionedExecutableModules executableOtherModules fromExecutableSection (fromExecutableSection [])
   . expandMain
@@ -1039,10 +1070,10 @@ expandMain = flatten . expand
       , sectionConditionals = map (fmap flatten) sectionConditionals
       }
 
-toSection :: CommonOptions CSources JsSources a -> WithCommonOptions CSources JsSources a -> Section a
+toSection :: CommonOptions CSources CxxSources JsSources a -> WithCommonOptions CSources CxxSources JsSources a -> Section a
 toSection globalOptions (Product options a) = toSection_ (Product (globalOptions <> options) a)
 
-toSection_ :: WithCommonOptions CSources JsSources a -> Section a
+toSection_ :: WithCommonOptions CSources CxxSources JsSources a -> Section a
 toSection_ (Product CommonOptions{..} a) = Section {
         sectionData = a
       , sectionSourceDirs = fromMaybeList commonOptionsSourceDirs
@@ -1054,6 +1085,8 @@ toSection_ (Product CommonOptions{..} a) = Section {
       , sectionCppOptions = fromMaybeList commonOptionsCppOptions
       , sectionCcOptions = fromMaybeList commonOptionsCcOptions
       , sectionCSources = commonOptionsCSources
+      , sectionCxxOptions = fromMaybeList commonOptionsCxxOptions
+      , sectionCxxSources = commonOptionsCxxSources
       , sectionJsSources = commonOptionsJsSources
       , sectionExtraLibDirs = fromMaybeList commonOptionsExtraLibDirs
       , sectionExtraLibraries = fromMaybeList commonOptionsExtraLibraries
@@ -1072,7 +1105,7 @@ toSection_ (Product CommonOptions{..} a) = Section {
   where
     conditionals = map toConditional (fromMaybeList commonOptionsWhen)
 
-    toConditional :: ConditionalSection CSources JsSources a -> Conditional (Section a)
+    toConditional :: ConditionalSection CSources CxxSources JsSources a -> Conditional (Section a)
     toConditional x = case x of
       ThenElseConditional (Product (ThenElse then_ else_) c) -> conditional c (toSection_ then_) (Just $ toSection_ else_)
       FlatConditional (Product sect c) -> conditional c (toSection_ sect) Nothing
