@@ -59,19 +59,18 @@ renderPackageWith :: RenderSettings -> Alignment -> [String] -> [(String, [Strin
 renderPackageWith settings headerFieldsAlignment existingFieldOrder sectionsFieldOrder Package{..} = intercalate "\n" (unlines header : chunks)
   where
     chunks :: [String]
-    chunks = map unlines . filter (not . null) . map (render settings 0) $ sortSectionFields sectionsFieldOrder stanzas
+    chunks = map unlines . filter (not . null) . map (render settings 0) $ sortStanzaFields sectionsFieldOrder stanzas
 
     header :: [String]
-    header = concatMap (render settings {renderSettingsFieldAlignment = headerFieldsAlignment} 0) (filterVerbatim packageVerbatim $ headerFields)
+    header = concatMap (render settings {renderSettingsFieldAlignment = headerFieldsAlignment} 0) packageFields
 
-    extraSourceFiles :: Element
-    extraSourceFiles = Field "extra-source-files" (LineSeparatedList packageExtraSourceFiles)
-
-    extraDocFiles :: Element
-    extraDocFiles = Field "extra-doc-files" (LineSeparatedList packageExtraDocFiles)
-
-    dataFiles :: Element
-    dataFiles = Field "data-files" (LineSeparatedList packageDataFiles)
+    packageFields :: [Element]
+    packageFields = addVerbatim packageVerbatim . sortFieldsBy existingFieldOrder $
+      headerFields ++ [
+        Field "extra-source-files" (LineSeparatedList packageExtraSourceFiles)
+      , Field "extra-doc-files" (LineSeparatedList packageExtraDocFiles)
+      , Field "data-files" (LineSeparatedList packageDataFiles)
+      ] ++ maybe [] (return . Field "data-dir" . Literal) packageDataDir
 
     sourceRepository :: [Element]
     sourceRepository = maybe [] (return . renderSourceRepository) packageSourceRepository
@@ -83,13 +82,9 @@ renderPackageWith settings headerFieldsAlignment existingFieldOrder sectionsFiel
     library = maybe [] (return . renderLibrary) packageLibrary
 
     stanzas :: [Element]
-    stanzas = addVerbatim packageVerbatim $
-      extraSourceFiles
-      : extraDocFiles
-      : dataFiles
-      : sourceRepository
-      ++ concat [
-        customSetup
+    stanzas = concat [
+        sourceRepository
+      , customSetup
       , map renderFlag packageFlags
       , library
       , renderInternalLibraries packageInternalLibraries
@@ -99,7 +94,7 @@ renderPackageWith settings headerFieldsAlignment existingFieldOrder sectionsFiel
       ]
 
     headerFields :: [Element]
-    headerFields = sortFieldsBy existingFieldOrder . mapMaybe (\(name, value) -> Field name . Literal <$> value) $ [
+    headerFields = mapMaybe (\(name, value) -> Field name . Literal <$> value) $ [
         ("name", Just packageName)
       , ("version", Just packageVersion)
       , ("synopsis", packageSynopsis)
@@ -148,6 +143,7 @@ renderPackageWith settings headerFieldsAlignment existingFieldOrder sectionsFiel
           makeVersion [1,22] <$ guard hasReexportedModules
         , makeVersion [2,0]  <$ guard hasSignatures
         , makeVersion [2,0] <$ guard hasGeneratedModules
+        , makeVersion [2,2] <$ guard (hasCxxParams sect)
         ]
         where
           hasReexportedModules = any (not . null . libraryReexportedModules) sect
@@ -155,19 +151,37 @@ renderPackageWith settings headerFieldsAlignment existingFieldOrder sectionsFiel
           hasGeneratedModules = any (not . null . libraryGeneratedModules) sect
 
       internalLibsCabalVersion :: Map String (Section Library) -> Maybe Version
-      internalLibsCabalVersion internalLibraries = makeVersion [2,0] <$ guard (not (Map.null internalLibraries))
+      internalLibsCabalVersion internalLibraries
+        | Map.null internalLibraries = Nothing
+        | otherwise = foldr max (Just $ makeVersion [2,0]) versions
+        where
+          versions = libraryCabalVersion <$> Map.elems internalLibraries
 
       executablesCabalVersion :: Map String (Section Executable) -> Maybe Version
       executablesCabalVersion = foldr max Nothing . map executableCabalVersion . Map.elems
 
       executableCabalVersion :: Section Executable -> Maybe Version
-      executableCabalVersion sect = makeVersion [2,0] <$ guard (executableHasGeneratedModules sect)
+      executableCabalVersion sect = maximum [
+          makeVersion [2,0] <$ guard (executableHasGeneratedModules sect)
+        , makeVersion [2,2] <$ guard (hasCxxParams sect)
+        ]
 
       executableHasGeneratedModules :: Section Executable -> Bool
       executableHasGeneratedModules = any (not . null . executableGeneratedModules)
 
-sortSectionFields :: [(String, [String])] -> [Element] -> [Element]
-sortSectionFields sectionsFieldOrder = go
+      hasCxxParams :: Section a -> Bool
+      hasCxxParams sect = or [
+          check sect
+        , any (any check) (sectionConditionals sect)
+        ]
+        where
+          check s = or [
+              (not . null . sectionCxxOptions) s
+            , (not . null . sectionCxxSources) s
+            ]
+
+sortStanzaFields :: [(String, [String])] -> [Element] -> [Element]
+sortStanzaFields sectionsFieldOrder = go
   where
     go sections = case sections of
       [] -> []
@@ -278,9 +292,11 @@ renderSection renderSectionData extraFieldsStart extraFieldsEnd Section{..} = ad
   , renderGhcjsOptions sectionGhcjsOptions
   , renderCppOptions sectionCppOptions
   , renderCcOptions sectionCcOptions
+  , renderCxxOptions sectionCxxOptions
   , renderDirectories "include-dirs" sectionIncludeDirs
   , Field "install-includes" (LineSeparatedList sectionInstallIncludes)
   , Field "c-sources" (LineSeparatedList sectionCSources)
+  , Field "cxx-sources" (LineSeparatedList sectionCxxSources)
   , Field "js-sources" (LineSeparatedList sectionJsSources)
   , renderDirectories "extra-lib-dirs" sectionExtraLibDirs
   , Field "extra-libraries" (LineSeparatedList sectionExtraLibraries)
@@ -387,6 +403,9 @@ renderCppOptions = Field "cpp-options" . WordList
 renderCcOptions :: [CcOption] -> Element
 renderCcOptions = Field "cc-options" . WordList
 
+renderCxxOptions :: [CxxOption] -> Element
+renderCxxOptions = Field "cxx-options" . WordList
+
 renderLdOptions :: [LdOption] -> Element
 renderLdOptions = Field "ld-options" . WordList
 
