@@ -37,6 +37,34 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
           Paths_foo
       |]
 
+    describe "data-files" $ do
+      it "accepts data-files" $ do
+        touch "data/foo/index.html"
+        touch "data/bar/index.html"
+        [i|
+        data-files:
+          - data/**/*.html
+        |] `shouldRenderTo` package [i|
+        data-files:
+            data/bar/index.html
+            data/foo/index.html
+        |]
+
+    describe "data-dir" $ do
+      it "accepts data-dir" $ do
+        touch "data/foo.html"
+        touch "data/bar.html"
+        [i|
+        data-dir: data
+        data-files:
+          - "*.html"
+        |] `shouldRenderTo` package [i|
+        data-files:
+            bar.html
+            foo.html
+        data-dir: data
+        |]
+
     describe "github" $ do
       it "accepts owner/repo" $ do
         [i|
@@ -440,6 +468,29 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             foo.js
             jsbits/bar.js
         |]
+
+    describe "cxx-options" $ do
+      it "accepts cxx-options" $ do
+        [i|
+        executable:
+          cxx-options: -Wall
+        |] `shouldRenderTo` (executable_ "foo" [i|
+        cxx-options: -Wall
+        |]) {packageCabalVersion = ">= 2.2"}
+
+    describe "cxx-sources" $ before_ (touch "foo.cc" >> touch "cxxbits/bar.cc") $ do
+      it "accepts cxx-sources" $ do
+        [i|
+        executable:
+          cxx-sources:
+            - foo.cc
+            - cxxbits/*.cc
+        |] `shouldRenderTo` (executable_ "foo" [i|
+        cxx-sources:
+            cxxbits/bar.cc
+            foo.cc
+        |]) {packageCabalVersion = ">= 2.2"}
+
     describe "extra-lib-dirs" $ do
       it "accepts extra-lib-dirs" $ do
         [i|
@@ -1160,11 +1211,11 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             foo: 23
           library: {}
           |] `shouldRenderTo` package [i|
+          foo: 23
           library
             other-modules:
                 Paths_foo
             default-language: Haskell2010
-          foo: 23
           |]
 
       context "within a section" $ do
