@@ -1,3 +1,5 @@
+{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE ConstraintKinds #-}
 module Helper (
   module Test.Hspec
 , module Test.Mockery.Directory
@@ -5,6 +7,7 @@ module Helper (
 , withTempDirectory
 , module System.FilePath
 , withCurrentDirectory
+, yaml
 ) where
 
 import           Test.Hspec
@@ -15,6 +18,9 @@ import           Control.Exception
 import qualified System.IO.Temp as Temp
 import           System.FilePath
 
+import           Data.Yaml.TH (yamlQQ)
+import           Language.Haskell.TH.Quote (QuasiQuoter)
+
 withCurrentDirectory :: FilePath -> IO a -> IO a
 withCurrentDirectory dir action = do
   bracket getCurrentDirectory setCurrentDirectory $ \ _ -> do
@@ -24,3 +30,6 @@ withCurrentDirectory dir action = do
 withTempDirectory :: (FilePath -> IO a) -> IO a
 withTempDirectory action = Temp.withSystemTempDirectory "hspec" $ \dir -> do
   canonicalizePath dir >>= action
+
+yaml :: Language.Haskell.TH.Quote.QuasiQuoter
+yaml = yamlQQ
