@@ -1,14 +1,8 @@
-{-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE QuasiQuotes #-}
 module Hpack.UtilSpec (main, spec) where
 
-import           Data.Aeson
-import           Data.Aeson.QQ
-import           Data.Aeson.Types
 import           Helper
 import           System.Directory
 
-import           Hpack.Config
 import           Hpack.Util
 
 main :: IO ()
@@ -71,28 +65,6 @@ spec = do
           touch "foo/bar/baz"
           getModuleFilesRecursive "foo" `shouldReturn` empty
 
-  describe "List" $ do
-    let invalid = [aesonQQ|{
-          name: "hpack",
-          gi: "sol/hpack",
-          ref: "master"
-        }|]
-        parseError :: String -> Either String (List Dependency)
-        parseError prefix = Left (prefix ++ ": neither key \"git\" nor key \"github\" present")
-    context "when parsing single values" $ do
-      it "returns the value in a singleton list" $ do
-        fromJSON (toJSON $ Number 23) `shouldBe` Success (List [23 :: Int])
-
-      it "returns error messages from element parsing" $ do
-        parseEither parseJSON invalid `shouldBe` parseError "Error in $"
-
-    context "when parsing a list of values" $ do
-      it "returns the list" $ do
-        fromJSON (toJSON [Number 23, Number 42]) `shouldBe` Success (List [23, 42 :: Int])
-
-      it "propagates parse error messages of invalid elements" $ do
-        parseEither parseJSON (toJSON [String "foo", invalid]) `shouldBe` parseError "Error in $[1]"
-
   describe "tryReadFile" $ do
     it "reads file" $ do
       inTempDirectory $ do
@@ -105,8 +77,8 @@ spec = do
 
   describe "expandGlobs" $ around withTempDirectory $ do
     it "accepts simple files" $ \dir -> do
-        touch (dir </> "foo.js")
-        expandGlobs "field-name" dir ["foo.js"] `shouldReturn` ([], ["foo.js"])
+      touch (dir </> "foo.js")
+      expandGlobs "field-name" dir ["foo.js"] `shouldReturn` ([], ["foo.js"])
 
     it "removes duplicates" $ \dir -> do
       touch (dir </> "foo.js")
@@ -156,11 +128,15 @@ spec = do
 
     context "when a pattern does not match anything" $ do
       it "warns" $ \dir -> do
-        expandGlobs "field-name" dir ["foo"] `shouldReturn`
-          (["Specified pattern \"foo\" for field-name does not match any files"], [])
+        expandGlobs "field-name" dir ["*.foo"] `shouldReturn`
+          (["Specified pattern \"*.foo\" for field-name does not match any files"], [])
 
     context "when a pattern only matches a directory" $ do
       it "warns" $ \dir -> do
         createDirectory (dir </> "foo")
-        expandGlobs "field-name" dir ["foo"] `shouldReturn`
-          (["Specified pattern \"foo\" for field-name does not match any files"], [])
+        expandGlobs "field-name" dir ["fo?"] `shouldReturn`
+          (["Specified pattern \"fo?\" for field-name does not match any files"], [])
+
+    context "when a literal file does not exist" $ do
+      it "warns and keeps the file" $ \dir -> do
+        expandGlobs "field-name" dir ["foo.js"] `shouldReturn` (["Specified file \"foo.js\" for field-name does not exist"], ["foo.js"])
